/*
8*8*8 LED-Cube
Maturaarbeit von Peter Werner am Gymnasium Kirchenfeld http://www.gymkirchenfeld.ch/
08.12.14
Disclaimer:
Any mofos selling this program will be sued...jk. 

*/
#include <SPI.h>

#define clock_pin 13 //SCK muss dieser Pin sein
#define data_pin 11 //MOSI " " " "
#define latch_pin 12 // kann irgendein Pin sein


volatile int count = 0;
volatile int layer = 0;
volatile int state = LOW;

byte outputbuffer[64];
byte data[512];


void setup(){

  for(int i=0;i<64;i++){
    outputbuffer[i]=0;
  }
  for(int i=0;i<512;i++){
    data[i]=0;
  }

  cli();

  pinMode(clock_pin,OUTPUT);
  pinMode(data_pin,OUTPUT);
  pinMode(latch_pin,OUTPUT);

  digitalWrite(latch_pin,LOW);//sicherstellen,dass die Latchlinie und die Clocklinie auf 0 initialisiert sind, da fehler aufgetreten sind
  digitalWrite(clock_pin,LOW);

  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);// Mode 0 Rising edge of data, keep clock low
  SPI.setClockDivider(SPI_CLOCK_DIV2);//Run the data in at 16MHz/2 - 8MHz

  //Timerinterrupt code von : http://www.instructables.com/id/Arduino-Timer-Interrupts/?ALLSTEPS (21.11.14)

  TCCR1A = 0;// set entire TCCR1A register to 0
  TCCR1B = 0;// same for TCCR1B
  TCNT1  = 0;//initialize counter value to 0
  // set compare match register for 1hz increments
  OCR1A = 1;// = (16*10^6) / (1*1024) - 1 (must be <65536)
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  // Set CS10 and CS12 bits for 1024 prescaler
  TCCR1B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);
  //SPI.begin();
  sei();


}

void loop(){
  test();
  expandingBall();
  slidingcubes();
  bouncingcube();
  sinwave();
  bouncingplanes();
  intersectboucingplanes();
}

//Cubeprogramm--------------------------------------------------------------------------------
void updatebuffer(){
  for (int ii=0; ii < 64; ii++){
    byte currentbyte =0;
    for (int i=0; i<8; i++){
      currentbyte = currentbyte + data[ii*8 + 7 - i] * (1 << i);
    }
    outputbuffer[63-ii]= currentbyte;
  }
}


ISR(TIMER1_COMPA_vect){
  digitalWrite(latch_pin,LOW);  
  SPI.begin();
  SPI.transfer(128>>layer); //Transistoren Daten-> Q0 auf dem letzten Schieberegister ist die unterste Ebene
  for (int i = 0; i < 8; ++i) {
    SPI.transfer(outputbuffer[i+layer*8]);
  }
  SPI.end();
  digitalWrite(latch_pin,HIGH);
  //PORTD |= 1<<latch_pin;//Latch pin HIGH
  ++layer;
  if (layer > 7) {  
    layer = 0;
  }
}

//----------------------------------------------------------------------------------------------

//setzt alle werte im Daten array auf 0
void clearData(){
  for(int i;i<512;i++){
    data[i]=0;
  }
}

//Wandelt x,y,z Koordinaten in den Index des Datenarrays um
int KzI(int x, int y,int z){ 
  int N=x+y*8+z*64;
  return N;
}

//Wandelt Index in Koordinaten um
void IzK(int ix, int *x, int *y, int *z){
  *z = ix/64;
  *y = (ix - *z*64)/8;
  *x = ix - *z*64 - *y*8;
}
//Animationen-------------------------------------------------------------
void test(){ 
  clearData();
  updatebuffer();
  for(int u=0;u<512;u++){
    //    data[KzI(3,3,3)]=1;
    data[u] = 1;
    updatebuffer();
    delay(5);
  } 
  clearData();
}
//--------------------------------------------------------------------------
void slidingcubes(){
  int motion[8]={ 
    0,1,2,3,4,3,2,1    };
  for(int a=0;a<8;a++){
    for(int u=0;u<8;u++){
      for(int i=0;i<4;i++){
        for(int ii=0;ii<4;ii++){
          for(int iii=0;iii<4;iii++){
            data[KzI(i,ii,iii+motion[u])]=1;
            data[KzI(i+4,ii+4,iii+motion[u])]=1;
            data[KzI(i+4,ii,iii+4-motion[u])]=1;
            data[KzI(i,ii+4,iii+4-motion[u])]=1;
          }
        }
      }
      updatebuffer();
      delay(10);
      clearData();
    }
  }
}

//----------------------------------------------------------------------------------------------------------------------------------
void bouncingplanes(){
  for(int u=0;u<3;u++){  
    clearData();
    int motionx[43] = {
      0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,5,4,3,2,1,0,0,0,0,0,0,0,0         };//2nd
    int motiony[43] = {
      0,1,2,3,4,5,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,5,4,3,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0         };//1st
    int motionz[43] = {
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,5,4,3,2,1,0         };//3nd
    for(int Counter=0;Counter<43;Counter++){
      for(int x=0; x<8;x++){
        for(int y=0; y<8;y++){
          data[KzI(x,y,motionz[Counter])]=1;
        }
      }
      for(int i=0; i<8;i++){
        for(int ii=0; ii<8;ii++){
          data[KzI(i,motiony[Counter],ii)]=1;
        }
      }
      for(int i=0; i<8;i++){
        for(int ii=0; ii<8;ii++){
          data[KzI(motionx[Counter],i,ii)]=1;
        }
      }
      updatebuffer();
      delay(100);
      clearData();
    }
  }
}

//----------------------------------------------------------------------------------------------------------------------------------
void intersectboucingplanes(){  
  clearData();
  int motion[15] = {
    0,1,2,3,4,5,6,7,6,5,4,3,2,1,0    };  
  for(int u=0;u<12;u++){ 
    for(int Counter=0;Counter<14;Counter++){
      for(int i=0; i<8;i++){
        for(int ii=0; ii<8;ii++){
          data[KzI(i,ii,motion[Counter])]=1;
        }
      }
      if(u>3){
        for(int i=0; i<8;i++){
          for(int ii=0; ii<8;ii++){
            data[KzI(i,motion[Counter],ii)]=1;
          }
        }
      }
      if(u>7){
        for(int i=0; i<8;i++){
          for(int ii=0; ii<8;ii++){
            data[KzI(motion[Counter],i,ii)]=1;
          }

        }
      }
      updatebuffer();
      delay(100);
      clearData();
    }
  }
}

//----------------------------------------------------------------------------------------------------------------------------------

void bouncingcube(){
  clearData();
  int iterations;
  float vx,vy,vz,x = 3,y=3,z=3;
  //vx=random(-200, 200)/100 + 1;
  vx=1;
  //vy=random(-200, 200)/100 + 1;
  vy=-1;
  //vz=random(-200, 200)/100 + 1;
  vz=-0.5;

  iterations=300;

  data[KzI((int)x,(int)y,(int)z)]=1;
  data[KzI((int)x+1,(int)y,(int)z)]=1;
  data[KzI((int)x+1,(int)y+1,(int)z)]=1;
  data[KzI((int)x,(int)y+1,(int)z)]=1;

  data[KzI((int)x,(int)y,(int)z+1)]=1;
  data[KzI((int)x+1,(int)y,(int)z+1)]=1;
  data[KzI((int)x+1,(int)y+1,(int)z+1)]=1;
  data[KzI((int)x,(int)y+1,(int)z+1)]=1;
  updatebuffer();
  delay(200);
  for(int i=0;i<iterations;i++){
    updatebuffer();
    delay(10);
    clearData();
    x=x+vx;
    y=y+vy;
    z=z+vz;
    if(x>6){
      vx=-vx;
      x=x+2*vx;
    }
    if(x<0){
      vx=-vx;
      x=x+vx;
    }
    if(y>6){
      vy=-vy;
      y=y+2*vy;
    }
    if(y<0){
      vy=-vy;
      y=y+2*vy;
    }
    if(z>6){
      vz=-vz;
      z=z+2*vz;
    }
    if(z<0){
      vz=-vz;
      z=z+2*vz;
    }
    data[KzI((int)x,(int)y,(int)z)]=1;
    data[KzI((int)x+1,(int)y,(int)z)]=1;
    data[KzI((int)x+1,(int)y+1,(int)z)]=1;
    data[KzI((int)x,(int)y+1,(int)z)]=1;

    data[KzI((int)x,(int)y,(int)z+1)]=1;
    data[KzI((int)x+1,(int)y,(int)z+1)]=1;
    data[KzI((int)x+1,(int)y+1,(int)z+1)]=1;
    data[KzI((int)x,(int)y+1,(int)z+1)]=1;
  }
}

//----------------------------------------------------------
void expandingBall(){
  clearData();
  float cx=3.5;
  float cy=3.5;
  float cz=3.5;
  float radius=0.5;
  float vr=0.5;
 while(radius<4){
  for(int x=0;x<8;x++){
    for(int y=0;y<8;y++){
      for(int z=0;z<8;z++){
        float distance;
        float fx=x,fy=y,fz=z;
        float q =(cx-fx)*(cx-fx)+(cy-fy)*(cy-fy)+(cz-fz)*(cz-fz);
        distance = (float)sqrt(q);
        if(distance<=radius){
        data[KzI(x,y,z)]=1;
        }else{
        data[KzI(x,y,z)]=0;
        }
      }
    }
  }
  updatebuffer;
  delay(10);
  radius=radius+vr;
  }
  while(radius>0.5){
  for(int x=0;x<8;x++){
    for(int y=0;y<8;y++){
      for(int z=0;z<8;z++){
        float distance;
        float fx=x,fy=y,fz=z;
        float q =(cx-fx)*(cx-fx)+(cy-fy)*(cy-fy)+(cz-fz)*(cz-fz);
        distance = (float)sqrt(q);
        if(distance<=radius){
        data[KzI(x,y,z)]=1;
        }else{
        data[KzI(x,y,z)]=0;
        }
      }
    }
  }
  updatebuffer;
  delay(10);
  radius=radius-vr;
  }
}
//------------------------------------------------------------------------------------
void sinwave(){
  float centerx=3.5;
  float centery=3.5;
  float centerz=3.5;
  float wavesize=3.5;
  float iterationsteps=20;//speed
  int iterations=320; //length of animation
  float wl=0.25;//wavelength in cube

  for(int counter=0;counter<iterations;counter++){
    clearData();
    for(int x=0;x<8;x++){
      for(int y=0;y<8;y++){
        float distance=sqrt(((float)x-centerx)*((float)x-centerx)+((float)y-centery)*((float)y-centery));
        int zcord= (int)(3.5+wavesize*sin((3.14159/iterationsteps)*counter+wl*distance)); //sqrt formate vertragen sich nicht
        data[KzI(x,y,zcord)]=1;
      }
    }
    updatebuffer();
    delay(100);
    clearData();
  }
}





